"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addDataConfigToKeplerGl = addDataConfigToKeplerGl;
exports.dataToDatasets = dataToDatasets;
exports["default"] = void 0;
var _actions = require("@kepler.gl/actions");
var _schemas = require("@kepler.gl/schemas");
var _document = _interopRequireDefault(require("global/document"));
var _root = _interopRequireDefault(require("./components/root"));
var _store = _interopRequireDefault(require("./store"));
var _utils = require("./utils");
var _log = _interopRequireDefault(require("../log"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _classCallCheck(a, n) { if (!(a instanceof n)) throw new TypeError("Cannot call a class as a function"); }
function _defineProperties(e, r) { for (var t = 0; t < r.length; t++) { var o = r[t]; o.enumerable = o.enumerable || !1, o.configurable = !0, "value" in o && (o.writable = !0), Object.defineProperty(e, _toPropertyKey(o.key), o); } }
function _createClass(e, r, t) { return r && _defineProperties(e.prototype, r), t && _defineProperties(e, t), Object.defineProperty(e, "prototype", { writable: !1 }), e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); } // SPDX-License-Identifier: MIT
// Copyright contributors to the kepler.gl project
var getData = function getData(that) {
  return that.model.get('data');
};
var getConfig = function getConfig(that) {
  return that.model.get('config');
};
var getHeight = function getHeight(that) {
  return that.model.get('height');
};
var DOM_EL_ID = 'keplergl';
var counter = 0;
var NONE_UPDATE_ACTIONS = [_actions.ActionTypes.REGISTER_ENTRY, _actions.ActionTypes.DELETE_ENTRY, _actions.ActionTypes.RENAME_ENTRY, _actions.ActionTypes.LOAD_MAP_STYLES, _actions.ActionTypes.LAYER_HOVER];
function getConfigInStore() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
    _ref$hash = _ref.hash,
    hash = _ref$hash === void 0 ? true : _ref$hash,
    store = _ref.store;
  if (store) {
    var currentState = store.getState().keplerGl.map;
    var currentValue = _schemas.KeplerGlSchema.getConfigToSave(currentState);
    return hash ? JSON.stringify(currentValue) : currentValue;
  }
  return {};
}
function getDatasetsInStore(store) {
  if (store) {
    return store.getState().keplerGl.map.visState.datasets;
  }
}
var KeplerGlJupyter = /*#__PURE__*/function () {
  function KeplerGlJupyter() {
    _classCallCheck(this, KeplerGlJupyter);
    this.id = "".concat(DOM_EL_ID, "-").concat(counter);
    counter++;
    this.mapUpdateCounter = 0;
  }
  return _createClass(KeplerGlJupyter, [{
    key: "create",
    value: function create(that) {
      (0, _log["default"])('kepler.gl create');
      var previousValue;
      function handleStoreChange(action, nextStore) {
        (0, _log["default"])(action);
        if (!action || NONE_UPDATE_ACTIONS.includes(action.type)) {
          return;
        }
        var saveState = getConfigInStore({
          hash: false,
          store: nextStore
        });
        var hash = JSON.stringify(saveState);

        // should not update model after first UPDATE_MAP action
        // when component first mounted
        if (previousValue !== hash && this.mapUpdateCounter > 2) {
          // keplerGl State has changed
          (0, _log["default"])('store state has changed, update model');
          (0, _log["default"])(previousValue);
          (0, _log["default"])(hash);
          previousValue = hash;
          that.model.set({
            config: saveState
          });

          // that.model.save_changes();
          that.touch();
        }
        if (action.type === _actions.ActionTypes.UPDATE_MAP) {
          this.mapUpdateCounter++;
        }
      }
      this.store = (0, _store["default"])(handleStoreChange.bind(this));
      var height = getHeight(that);
      that.el.classList.add('jupyter-widgets');
      that.el.classList.add('keplergl-jupyter-widgets');
      var divElmt = _document["default"].createElement('div');
      divElmt.setAttribute('id', this.id);
      divElmt.classList.add('kepler-gl');
      divElmt.setAttribute('style', " width: 100%; height: ".concat(height, "px;"));
      that.el.appendChild(divElmt);
      (0, _root["default"])({
        id: this.id,
        store: this.store,
        ele: divElmt
      });
      var data = getData(that);
      var config = getConfig(that);
      (0, _log["default"])('<<<<<<<< render finished! >>>>>>>>>');

      // After rendering the component,
      // we add the data that's already in the model
      var hasData = data && Object.keys(data).length;
      var hasConfig = config && config.version;
      if (hasData) {
        (0, _log["default"])('data already in model');
        addDataConfigToKeplerGl({
          data: data,
          config: config,
          store: this.store
        });
      } else if (hasConfig) {
        (0, _log["default"])('config already in model');
        this.onConfigChange(that);
      }
    }
  }, {
    key: "onDataChange",
    value: function onDataChange(that) {
      (0, _log["default"])('kepler.gl onDataChange');
      var data = getData(that);
      addDataConfigToKeplerGl({
        data: data,
        store: this.store
      });
    }
  }, {
    key: "onConfigChange",
    value: function onConfigChange(that) {
      (0, _log["default"])('kepler.gl onConfigChange');
      var config = getConfig(that);
      var currentValue = getConfigInStore({
        hash: true,
        store: this.store
      });
      if (currentValue === JSON.stringify(config)) {
        // calling model.set('config') inside the js component will trigger another onConfigChange
        (0, _log["default"])('onConfigChange: config is the same as saved in store');
        return;
      }
      this.store.dispatch((0, _actions.addDataToMap)({
        // reuse datasets in state
        // a hack to apply config to existing data
        datasets: Object.values(getDatasetsInStore(this.store)).map(function (d) {
          return {
            info: {
              id: d.id,
              label: d.label,
              color: d.color
            },
            data: _objectSpread({
              fields: d.fields
            }, d.dataContainer instanceof ArrowDataContainer ? {
              cols: d.dataContainer._cols
            } : {
              rows: d.allData
            })
          };
        }),
        config: config,
        options: {
          centerMap: false
        }
      }));
    }
  }]);
}();
function addDataConfigToKeplerGl(_ref2) {
  var inputData = _ref2.data,
    config = _ref2.config,
    options = _ref2.options,
    store = _ref2.store;
  var data = inputData ? dataToDatasets(inputData) : [];
  (0, _log["default"])(data);
  var results = (0, _utils.loadJupyterData)(data);
  var succeeded = results.filter(function (r) {
    return r && r.data;
  });
  (0, _log["default"])('addDataConfigToKeplerGl');
  (0, _log["default"])(succeeded);
  (0, _log["default"])(config);
  var hasMapState = Boolean(config && config.config && config.config.mapState);
  store.dispatch((0, _actions.addDataToMap)({
    datasets: succeeded,
    config: config,
    options: options || {
      centerMap: !hasMapState
    }
  }));
}
function dataToDatasets(data) {
  return Object.keys(data).map(function (key) {
    return {
      id: key,
      data: data[key]
    };
  });
}
var _default = exports["default"] = KeplerGlJupyter;