"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadJupyterData = loadJupyterData;
var _apacheArrow = require("apache-arrow");
var _processors = require("@kepler.gl/processors");
var _log = _interopRequireDefault(require("../log"));
var _console = _interopRequireDefault(require("global/console"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); } // SPDX-License-Identifier: MIT
// Copyright contributors to the kepler.gl project
function handleJuptyerDataFormat(dataEntry) {
  // This makes passing data between Jupyter the iframe easier
  // detect data type here
  (0, _log["default"])('handleJuptyerDataFormat');
  var data = dataEntry.data,
    id = dataEntry.id;
  var parsed = data;
  var type = 'csv';
  if (_typeof(data) === 'object') {
    if (data.columns && data.data && data.index) {
      // Data is parsed as a Dataframe
      (0, _log["default"])('data is a dataframe');
      type = 'df';
      // parsed = {fields: data.columns, data: data.data};
    } else {
      // assume is geojson
      type = 'json';
    }
  } else if (typeof data === 'string') {
    // check if js string is json string
    try {
      parsed = JSON.parse(data);
      type = 'json';
    } catch (e) {
      // assume it is base64 string represents arrow table
      try {
        _console["default"].log('parse base64string arrow tabl');
        // convert arrowTable from base64 string to ArrayBuffer
        var arrowTableBuffer = Buffer.from(data, 'base64').buffer;
        // create arrow table from ArrayBuffer
        parsed = (0, _apacheArrow.tableFromIPC)([new Uint8Array(arrowTableBuffer)]);
        type = 'arrow';
      } catch (e) {
        // now we can assume it is csv
      }
    }
  }
  return {
    data: parsed,
    type: type,
    id: id
  };
}
function processReceivedData(_ref) {
  var data = _ref.data,
    info = _ref.info;
  // assume there is only 1 file
  (0, _log["default"])('processReceivedData');
  var processed;
  try {
    processed = info.queryType === 'csv' ? (0, _processors.processCsvData)(data) : info.queryType === 'json' ? (0, _processors.processGeojson)(data) : info.queryType === 'df' ? processDataFrame(data) : info.queryType === 'arrow' ? (0, _processors.processArrowBatches)(data.batches) : null;
  } catch (e) {
    _console["default"].log("Kepler.gl fails to parse data, detected data\n    format is ".concat(info.queryType), e);
  }
  return {
    data: processed,
    info: info
  };
}
function processDataFrame(data) {
  var fields = data.columns.map(function (name) {
    return {
      name: name
    };
  });
  var rows = data.data;

  // kepler.gl will detect field types
  return {
    fields: fields,
    rows: rows
  };
}
function loadJupyterData(rawData) {
  var dataToLoad = rawData.map(handleJuptyerDataFormat).map(function (rd) {
    return {
      data: rd.data,
      info: {
        id: rd.id,
        label: rd.id,
        queryType: rd.type,
        queryOption: 'jupyter'
      }
    };
  });
  return dataToLoad.map(processReceivedData);
}